/*
Copyright (c) 2014 DenshiKousakuSenka
 This software is released under the MIT License.
 http://opensource.org/licenses/mit-license.php
 */

#pragma once

#include "DKS_I2C_Base.h"

////////////////////////
////////////  waitpmF
////////////////////////////

namespace DKS
{
	class MPU6050
	{
		const DKS::I2C::II2C *i2c;
		uint8_t slaveAddr;

		//full scale range of the gyroscope outputs
		enum FS_SEL
		{
			GYRO_FS_250 	= 0b00000,
			GYRO_FS_500 	= 0b01000,
			GYRO_FS_1000  	= 0b10000,
			GYRO_FS_2000	= 0b11000
		};

		//full scale range of the accelerometer
		enum AFS_SEL
		{
			ACCEL_FS_2  = 0b00000,
			ACCEL_FS_4  = 0b01000,
			ACCEL_FS_8  = 0b10000,
			ACCEL_FS_16 = 0b11000
		};

		enum LowPassFilter
		{
			DLPF_BW_256 = 0x00,	// Digital low pass filter bandwidth = 256Hz
			DLPF_BW_188 = 0x01,
			DLPF_BW_98  = 0x02,
			DLPF_BW_42  = 0x03,
			DLPF_BW_20  = 0x04,
			DLPF_BW_10  = 0x05,
			DLPF_BW_5   = 0x06
		};

		float rateA;
		float rateG;
	public:
		// DeviceAddress = 0b110_100X     X=1 (PinAD0 is high), X=0 (D0:Pin AD0 is low)
		MPU6050(const DKS::I2C::II2C *I2Cx, const bool PinAD0isHigh);

		bool TestConnection()const;
		uint8_t GetDeviceAddress()const;

		void Init(void);
		void setFullScaleGyroRange(const FS_SEL scale);
		void setFullScaleAccelRange(const AFS_SEL scale);

		// When DLPF== DLPF_BW_256, Sample Rate is not same with others.
		void setDigitalLowpassFilter(const LowPassFilter DLPF)const;

		// Sample Rate = Gyroscope Output Rate / (1 + SMPLRT_DIV)
		void setSampleRateDivider(const uint8_t divider)const;
		void getSampleRateDivider(uint8_t &divider)const;

		void GetData(int16_t Acc[3], int16_t &Temp, int16_t Gyro[3])const;
		void GetData(float Acc[3], float &Temp, float Gryo[3])const;
	};
}
